@@ -25,6 +25,10 @@ use std::{
     sync,
 };
 
+mod trust_set;
+
+pub use trust_set::*;
+
 #[derive(thiserror::Error, Debug)]
 pub enum Error {
     #[error("Unknown proof type '{}'", _0)]
@@ -224,6 +228,13 @@ impl AlternativesData {
     }
 }
 
+pub type TimestampedTrustDetails = Timestamped<TrustDetails>;
+#[derive(Debug, Clone)]
+pub struct TrustDetails {
+    level: TrustLevel,
+    override_: HashSet<Id>,
+}
+
 /// In memory database tracking information from proofs
 ///
 /// After population, used for calculating the effective trust set, etc.
@@ -234,7 +245,12 @@ impl AlternativesData {
 /// of some kind.
 pub struct ProofDB {
     /// who -(trusts)-> whom
-    trust_id_to_id: HashMap<Id, HashMap<Id, TimestampedTrustLevel>>,
+    trust_id_to_id: HashMap<Id, HashMap<Id, TimestampedTrustDetails>>,
+    /// who -(is being trusted by -> whom
+    reverse_trust_id_to_id: HashMap<Id, HashMap<Id, TimestampedTrustLevel>>,
+
+    /// (source, target) -> -(trust via)-> trust proof certificate
+    ids_to_trust_proof_signatures: HashMap<(Id, Id), TimestampedSignature>,
 
     /// Id->URL mapping verified by Id's signature
     /// boolean is whether it's been fetched from the same URL, or local trusted repo,
@@ -247,6 +263,9 @@ pub struct ProofDB {
     // all reviews are here
     package_review_by_signature: HashMap<Signature, review::Package>,
 
+    // all trust proofs here
+    trust_proofs_by_signature: HashMap<Signature, proof::Trust>,
+
     // we can get the to the review through the signature from these two
     package_review_signatures_by_package_digest:
         HashMap<Vec<u8>, HashMap<PkgVersionReviewId, TimestampedSignature>>,
@@ -258,6 +277,9 @@ pub struct ProofDB {
 
     package_flags: HashMap<proof::PackageId, HashMap<Id, TimestampedFlags>>,
 
+    // given an Id of an author, get the list of all package version id that were produced by it
+    from_id_to_package_reviews: HashMap<Id, HashSet<proof::PackageVersionId>>,
+
     // original data about pkg alternatives
     // for every package_id, we store a map of ids that had alternatives for it,
     // and a timestamped signature of the proof, so we keep track of only
@@ -277,6 +299,9 @@ impl Default for ProofDB {
     fn default() -> Self {
         ProofDB {
             trust_id_to_id: default(),
+            reverse_trust_id_to_id: default(),
+            ids_to_trust_proof_signatures: default(),
+            trust_proofs_by_signature: default(),
             url_by_id_self_reported: default(),
             url_by_id_reported_by_others: default(),
             package_review_signatures_by_package_digest: default(),
@@ -285,6 +310,7 @@ impl Default for ProofDB {
             package_reviews: default(),
             package_alternatives: default(),
             package_flags: default(),
+            from_id_to_package_reviews: default(),
 
             insertion_counter: 0,
             derived_alternatives: sync::RwLock::new(AlternativesData::new()),
@@ -803,6 +829,11 @@ impl ProofDB {
             .and_modify(|s| s.update_to_more_recent(&timestamp_signature))
             .or_insert_with(|| timestamp_signature.clone());
 
+        self.from_id_to_package_reviews
+            .entry(review.common.from.id.clone())
+            .or_default()
+            .insert(pkg_review_id.package_version_id.clone());
+
         self.package_reviews
             .entry(review.package.id.id.source.clone())
             .or_default()
@@ -870,21 +901,62 @@ impl ProofDB {
         proofs
     }
 
-    fn add_trust_raw(&mut self, from: &Id, to: &Id, date: DateTime<Utc>, trust: TrustLevel) {
-        let tl = TimestampedTrustLevel { value: trust, date };
+    fn add_trust_raw(
+        &mut self,
+        from: &Id,
+        to: &Id,
+        date: DateTime<Utc>,
+        trust_proof: &proof::Trust,
+        signature: &str,
+    ) {
+        let trust = TrustDetails {
+            level: trust_proof.trust,
+            override_: trust_proof
+                .override_
+                .iter()
+                .map(|o| o.id.id.clone())
+                .collect(),
+        };
+
+        let tl = TimestampedTrustLevel {
+            value: trust.level,
+            date,
+        };
+        let td = TimestampedTrustDetails { value: trust, date };
+
+        self.trust_proofs_by_signature
+            .insert(signature.to_owned(), trust_proof.to_owned());
+
+        let signature = TimestampedSignature {
+            value: signature.to_owned(),
+            date,
+        };
+
+        self.ids_to_trust_proof_signatures
+            .entry((from.to_owned(), to.to_owned()))
+            .and_modify(|e| e.update_to_more_recent(&signature))
+            .or_insert_with(|| signature);
+
         self.trust_id_to_id
             .entry(from.to_owned())
             .or_insert_with(HashMap::new)
             .entry(to.to_owned())
+            .and_modify(|e| e.update_to_more_recent(&td))
+            .or_insert_with(|| td);
+
+        self.reverse_trust_id_to_id
+            .entry(to.to_owned())
+            .or_insert_with(HashMap::new)
+            .entry(from.to_owned())
             .and_modify(|e| e.update_to_more_recent(&tl))
             .or_insert_with(|| tl);
     }
 
-    fn add_trust(&mut self, trust: &proof::Trust, fetched_from: FetchSource) {
+    fn add_trust(&mut self, trust: &proof::Trust, signature: &str, fetched_from: FetchSource) {
         let from = &trust.from();
         self.record_url_from_from_field(&trust.date_utc(), from, &fetched_from);
         for to in &trust.ids {
-            self.add_trust_raw(&from.id, &to.id, trust.date_utc(), trust.trust);
+            self.add_trust_raw(&from.id, &to.id, trust.date_utc(), trust, signature);
         }
         for to in &trust.ids {
             // Others should not be making verified claims about this URL,
@@ -902,6 +974,16 @@ impl ProofDB {
             .collect()
     }
 
+    pub fn get_reverse_trust_for<'id, 's: 'id>(
+        &'s self,
+        id: &'id Id,
+    ) -> impl Iterator<Item = (&'id Id, TrustLevel)> + 's {
+        self.reverse_trust_id_to_id
+            .get(id)
+            .into_iter()
+            .flat_map(|map| map.iter().map(|(id, trust_level)| (id, trust_level.value)))
+    }
+
     /// Get all Ids that authored a proof (with total count)
     pub fn all_author_ids(&self) -> BTreeMap<Id, usize> {
         let mut res = BTreeMap::new();
@@ -993,7 +1075,9 @@ impl ProofDB {
             proof::PackageReview::KIND => {
                 self.add_package_review(&proof.parse_content()?, proof.signature(), fetched_from)
             }
-            proof::Trust::KIND => self.add_trust(&proof.parse_content()?, fetched_from),
+            proof::Trust::KIND => {
+                self.add_trust(&proof.parse_content()?, proof.signature(), fetched_from)
+            }
             other => return Err(Error::UnknownProofType(other.into())),
         }
 
@@ -1009,172 +1093,42 @@ impl ProofDB {
         }
     }
 
-    fn get_trust_list_of_id(&self, id: &Id) -> impl Iterator<Item = (TrustLevel, &Id)> {
-        self.trust_id_to_id.get(id).map(|map| map.iter().map(|(id, trust)| (trust.value, id)))
-        .into_iter()
-        .flatten()
+    fn get_trust_details_list_of_id(&self, id: &Id) -> impl Iterator<Item = (&TrustDetails, &Id)> {
+        self.trust_id_to_id
+            .get(id)
+            .map(|map| map.iter().map(|(id, trust)| (&trust.value, id)))
+            .into_iter()
+            .flatten()
+    }
+
+    pub fn get_trust_proof_between(&self, from: &Id, to: &Id) -> Option<&proof::Trust> {
+        self.ids_to_trust_proof_signatures
+            .get(&(from.to_owned(), to.to_owned()))
+            .and_then(|sig| self.trust_proofs_by_signature.get(&sig.value))
+    }
+
+    fn get_package_reviews_by_author<'iter, 's: 'iter, 'id: 'iter>(
+        &'s self,
+        id: &'id Id,
+    ) -> impl Iterator<Item = &'s review::Package> + 'iter {
+        self.from_id_to_package_reviews
+            .get(id)
+            .into_iter()
+            .flat_map(move |set| {
+                set.iter()
+                    .map(move |package_version_id| PkgVersionReviewId {
+                        from: id.clone(),
+                        package_version_id: package_version_id.clone(),
+                    })
+            })
+            .map(move |pkg_version_review_id| {
+                &self.package_review_by_signature
+                    [&self.package_review_signatures_by_pkg_review_id[&pkg_version_review_id].value]
+            })
     }
 
     pub fn calculate_trust_set(&self, for_id: &Id, params: &TrustDistanceParams) -> TrustSet {
-        let mut distrusted = HashMap::new();
-
-        // We keep retrying the whole thing, with more and more
-        // distrusted Ids
-        loop {
-            let prev_distrusted_len = distrusted.len();
-            let trust_set = self.calculate_trust_set_internal(for_id, params, distrusted);
-            if trust_set.distrusted.len() <= prev_distrusted_len {
-                return trust_set;
-            }
-            distrusted = trust_set.distrusted;
-        }
-    }
-
-    /// Calculate the effective trust levels for IDs inside a WoT.
-    ///
-    /// This is one of the most important functions in `crev-wot`.
-    fn calculate_trust_set_internal(
-        &self,
-        for_id: &Id,
-        params: &TrustDistanceParams,
-        distrusted: HashMap<Id, DistrustedIdDetails>,
-    ) -> TrustSet {
-        /// Node that is to be visited
-        ///
-        /// Order of field is important, since we use the `Ord` trait
-        /// to visit nodes breadth-first with respect to trust level
-        #[derive(PartialOrd, Ord, Eq, PartialEq, Clone, Debug)]
-        struct Visit {
-            /// Effective transitive trust level of the node
-            effective_trust_level: TrustLevel,
-            /// Distance from the root, in some abstract numerical unit
-            distance: u64,
-            /// Id we're visit
-            id: Id,
-        }
-
-        let mut pending = BTreeSet::new();
-        let mut current_trust_set = TrustSet::default();
-        let initial_distrusted_len = distrusted.len();
-        current_trust_set.distrusted = distrusted;
-
-        pending.insert(Visit {
-            effective_trust_level: TrustLevel::High,
-            distance: 0,
-            id: for_id.clone(),
-        });
-        let mut previous_iter_trust_level = TrustLevel::High;
-        current_trust_set.record_trusted_id(for_id.clone(), for_id.clone(), 0, TrustLevel::High);
-
-        while let Some(current) = pending.iter().next().cloned() {
-            debug!("Traversing id: {:?}", current);
-            pending.remove(&current);
-
-            if current.effective_trust_level != previous_iter_trust_level {
-                debug!(
-                    "No more nodes with effective_trust_level of {}",
-                    previous_iter_trust_level
-                );
-                assert!(current.effective_trust_level < previous_iter_trust_level);
-                if initial_distrusted_len != current_trust_set.distrusted.len() {
-                    debug!("Some people got banned at the current trust level - restarting the WoT calculation");
-                    break;
-                }
-            } else {
-                previous_iter_trust_level = current.effective_trust_level;
-            }
-
-            for (direct_trust, candidate_id) in self.get_trust_list_of_id(&current.id) {
-                debug!(
-                    "{} ({}) reports trust level for {}: {}",
-                    current.id, current.effective_trust_level, candidate_id, direct_trust
-                );
-
-                if current_trust_set.is_distrusted(candidate_id) {
-                    debug!("{} is distrusted by {}", candidate_id, current.id);
-                    continue;
-                }
-
-                // Note: lower trust node can ban higher trust node, but only
-                // if it wasn't banned by a higher trust node beforehand.
-                // However banning by the same trust level node, does not prevent
-                // the node from banning others.
-                if direct_trust == TrustLevel::Distrust {
-                    debug!("Adding {} to distrusted list (via {})", candidate_id, current.id);
-                    // We discard the result, because we actually want to make as much
-                    // progress as possible before restaring building the WoT, and
-                    // we will not visit any node that was marked as distrusted,
-                    // becuse we check it for every node to be visited
-                    let _ = current_trust_set
-                        .record_distrusted_id(candidate_id.clone(), current.id.clone());
-
-                    continue;
-                }
-
-                // Note: we keep visiting nodes, even banned ones, just like they were originally
-                // reported
-                let effective_trust_level =
-                    std::cmp::min(direct_trust, current.effective_trust_level);
-                debug!(
-                    "Effective trust for {} {}",
-                    candidate_id, effective_trust_level
-                );
-
-                if effective_trust_level == TrustLevel::None {
-                    continue;
-                } else if effective_trust_level < TrustLevel::None {
-                    unreachable!(
-                        "this should not happen: candidate_effective_trust <= TrustLevel::None"
-                    );
-                }
-
-                let candidate_distance_from_current =
-                    if let Some(v) = params.distance_by_level(effective_trust_level) {
-                        v
-                    } else {
-                        debug!("Not traversing {}: trust too low", candidate_id);
-                        continue;
-                    };
-
-                let candidate_total_distance = current.distance + candidate_distance_from_current;
-
-                debug!(
-                    "Distance of {} from {}: {}. Total distance from root: {}.",
-                    candidate_id,
-                    current.id,
-                    candidate_distance_from_current,
-                    candidate_total_distance
-                );
-
-                if candidate_total_distance > params.max_distance {
-                    debug!(
-                        "Total distance of {}: {} higher than max_distance: {}.",
-                        candidate_id, candidate_total_distance, params.max_distance
-                    );
-                    continue;
-                }
-
-                if current_trust_set.record_trusted_id(
-                    candidate_id.clone(),
-                    current.id.clone(),
-                    candidate_total_distance,
-                    effective_trust_level,
-                ) {
-                    let visit = Visit {
-                        effective_trust_level,
-                        distance: candidate_total_distance,
-                        id: candidate_id.to_owned(),
-                    };
-                    if pending.insert(visit.clone()) {
-                        debug!("{:?} inserted for visit", visit);
-                    } else {
-                        debug!("{:?} alreading pending", visit);
-                    }
-                }
-            }
-        }
-
-        current_trust_set
+        TrustSet::from(self, for_id, params)
     }
 
     /// Finds which URL is the latest and claimed to belong to the given Id.
@@ -1238,128 +1192,6 @@ impl<'a> UrlOfId<'a> {
     }
 }
 
-/// Details of a one Id that is trusted
-#[derive(Debug, Clone)]
-pub struct TrustedIdDetails {
-    // distanc from the root of trust
-    pub distance: u64,
-    // effective, global trust from the root of the WoT
-    pub effective_trust_level: TrustLevel,
-    /// People that reported trust for this id
-    pub reported_by: HashMap<Id, TrustLevel>,
-}
-
-/// Details of a one Id that is distrusted
-#[derive(Debug, Clone, Default)]
-pub struct DistrustedIdDetails {
-    /// People that reported distrust for this id
-    pub reported_by: HashSet<Id>,
-}
-
-#[derive(Default, Debug, Clone)]
-pub struct TrustSet {
-    pub trusted: HashMap<Id, TrustedIdDetails>,
-    pub distrusted: HashMap<Id, DistrustedIdDetails>,
-}
-
-impl TrustSet {
-    pub fn trusted_ids(&self) -> impl Iterator<Item = &Id> {
-        self.trusted.keys()
-    }
-
-    pub fn is_trusted(&self, id: &Id) -> bool {
-        self.trusted.contains_key(id)
-    }
-
-    pub fn is_distrusted(&self, id: &Id) -> bool {
-        self.distrusted.contains_key(id)
-    }
-
-    /// Record that an Id is reported as distrusted
-    ///
-    /// Return `true` if it was previously considered as trusted,
-    /// and so that WoT traversal needs to be restarted
-    fn record_distrusted_id(&mut self, subject: Id, reported_by: Id) -> bool {
-        let res = self.trusted.remove(&subject).is_some();
-
-        self.distrusted
-            .entry(subject)
-            .or_default()
-            .reported_by
-            .insert(reported_by);
-
-        res
-    }
-
-    /// Record that an Id is reported as trusted
-    ///
-    /// Returns `true` if this actually added or changed the `subject` details,
-    /// which requires revising it's own downstream trusted Id details in the graph algorithm for it.
-    fn record_trusted_id(
-        &mut self,
-        subject: Id,
-        reported_by: Id,
-        distance: u64,
-        effective_trust_level: TrustLevel,
-    ) -> bool {
-        use std::collections::hash_map::Entry;
-
-        assert!(effective_trust_level >= TrustLevel::None);
-
-        match self.trusted.entry(subject) {
-            Entry::Vacant(entry) => {
-                let reported_by = vec![(reported_by, effective_trust_level)]
-                    .into_iter()
-                    .collect();
-                entry.insert(TrustedIdDetails {
-                    distance,
-                    effective_trust_level,
-                    reported_by,
-                });
-                true
-            }
-            Entry::Occupied(mut entry) => {
-                let mut changed = false;
-                let details = entry.get_mut();
-                if details.distance > distance {
-                    details.distance = distance;
-                    changed = true;
-                }
-                if details.effective_trust_level < effective_trust_level {
-                    details.effective_trust_level = effective_trust_level;
-                    changed = true;
-                }
-                match details.reported_by.entry(reported_by) {
-                    Entry::Vacant(entry) => {
-                        entry.insert(effective_trust_level);
-                        changed = true;
-                    }
-                    Entry::Occupied(mut entry) => {
-                        let level = entry.get_mut();
-                        if *level < effective_trust_level {
-                            *level = effective_trust_level;
-                            changed = true;
-                        }
-                    }
-                }
-                changed
-            }
-        }
-    }
-
-    pub fn get_effective_trust_level(&self, id: &Id) -> TrustLevel {
-        self.get_effective_trust_level_opt(id)
-            .unwrap_or(TrustLevel::None)
-    }
-
-    pub fn get_effective_trust_level_opt(&self, id: &Id) -> Option<TrustLevel> {
-        self.trusted
-            .get(id)
-            .map(|details| details.effective_trust_level)
-            .or_else(|| self.distrusted.get(id).map(|_| TrustLevel::Distrust))
-    }
-}
-
 pub struct TrustDistanceParams {
     pub max_distance: u64,
     pub high_trust_distance: u64,
@@ -1400,8 +1232,29 @@ impl Default for TrustDistanceParams {
     }
 }
 
+/// List of authors recommending override (ignore) trust / package review with their effective
+/// trust level.
+#[derive(Debug, Clone, Default)]
+pub struct OverrideSourcesDetails(HashMap<Id, TrustLevel>);
+
+impl OverrideSourcesDetails {
+    pub fn insert(&mut self, id: Id, level: TrustLevel) {
+        self.0
+            .entry(id)
+            .and_modify(|prev_level| *prev_level = level.max(*prev_level))
+            .or_insert(level);
+    }
+
+    pub fn max_level(&self) -> Option<TrustLevel> {
+        self.0.iter().map(|e| e.1).max().copied()
+    }
+}
+
 #[test]
 fn db_is_send_sync() {
     fn is<T: Send + Sync>() {}
     is::<ProofDB>();
 }
+
+#[cfg(test)]
+mod tests;
